@@ -58,6 +58,7 @@ class BaseSSHClient(object):
                  retry_delay=RETRY_DELAY,
                  allow_agent=True, timeout=None,
                  proxy_host=None,
+                 proxy_port=None,
                  _auth_thread_pool=True,
                  identity_auth=True):
         self._auth_thread_pool = _auth_thread_pool
@@ -76,13 +77,14 @@ class BaseSSHClient(object):
         self.allow_agent = allow_agent
         self.session = None
         self._host = proxy_host if proxy_host else host
+        self._port = proxy_port if proxy_port else self.port
         self.pkey = _validate_pkey_path(pkey, self.host)
         self.identity_auth = identity_auth
         self._keepalive_greenlet = None
         self._init()
 
     def _init(self):
-        self._connect(self._host, self.port)
+        self._connect(self._host, self._port)
         self._init_session()
         self._auth_retry()
         self._keepalive()
@@ -127,7 +129,7 @@ class BaseSSHClient(object):
             except Exception:
                 pass
         sleep(self.retry_delay)
-        self._connect(self._host, self.port, retries=retries)
+        self._connect(self._host, self._port, retries=retries)
         return self._init_session(retries=retries)
 
     def _connect(self, host, port, retries=1):
